@@ -153,9 +153,11 @@ gdk_event_queue_find_first (void)
 
   while (tmp_list)
     {
-      GdkEventPrivate *event = queued_events->data;
+      GdkEventPrivate *event = tmp_list->data;
       if (!(event->flags & GDK_EVENT_PENDING))
 	return tmp_list;
+
+      tmp_list = g_list_next (tmp_list);
     }
 
   return NULL;
@@ -376,12 +378,22 @@ expose_predicate (Display *display, XEvent *xevent, XPointer arg)
 {
   GdkExposeInfo *info = (GdkExposeInfo *)arg;
 
-  if (xevent->xany.type != Expose)
+  /* Compressing across GravityNotify events is safe, because
+   * we completely ignore them, so they can't change what
+   * we are going to draw. Compressing across GravityNotify
+   * events is necessay because during window-unshading animation
+   * we'll get a whole bunch of them interspersed with
+   * expose events.
+   */
+  if ((xevent->xany.type != Expose) && 
+      (xevent->xany.type != GravityNotify))
     {
       info->seen_nonmatching = TRUE;
     }
 
-  if (info->seen_nonmatching || (xevent->xany.window != info->window))
+  if (info->seen_nonmatching ||
+      (xevent->xany.type != Expose) ||
+      (xevent->xany.window != info->window))
     return FALSE;
   else
     return TRUE;
