@@ -205,7 +205,7 @@ gtk_button_class_init (GtkButtonClass *klass)
 static void
 gtk_button_init (GtkButton *button)
 {
-  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_FOCUS);
+  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_FOCUS | GTK_RECEIVES_DEFAULT);
   GTK_WIDGET_UNSET_FLAGS (button, GTK_NO_WINDOW);
 
   button->child = NULL;
@@ -636,7 +636,7 @@ static gint
 gtk_button_expose (GtkWidget      *widget,
 		   GdkEventExpose *event)
 {
-  GtkButton *button;
+  GtkButton *bin;
   GdkEventExpose child_event;
 
   g_return_val_if_fail (widget != NULL, FALSE);
@@ -645,14 +645,14 @@ gtk_button_expose (GtkWidget      *widget,
 
   if (GTK_WIDGET_DRAWABLE (widget))
     {
-      button = GTK_BUTTON (widget);
+      bin = GTK_BIN (widget);
       
       gtk_button_paint (widget, &event->area);
 
       child_event = *event;
-      if (GTK_BIN (button)->child && GTK_WIDGET_NO_WINDOW (GTK_BIN (button)->child) &&
-	  gtk_widget_intersect (GTK_BIN (button)->child, &event->area, &child_event.area))
-	gtk_widget_event (GTK_BIN (button)->child, (GdkEvent*) &child_event);
+      if (bin->child && GTK_WIDGET_NO_WINDOW (bin->child) &&
+	  gtk_widget_intersect (bin->child, &event->area, &child_event.area))
+	gtk_widget_event (bin->child, (GdkEvent*) &child_event);
     }
 
   return FALSE;
@@ -672,8 +672,6 @@ gtk_button_button_press (GtkWidget      *widget,
     {
       button = GTK_BUTTON (widget);
 
-      if (GTK_WIDGET_CAN_DEFAULT (widget) && (event->button == 1))
-	gtk_widget_grab_default (widget);
       if (!GTK_WIDGET_HAS_FOCUS (widget))
 	gtk_widget_grab_focus (widget);
 
