@@ -92,7 +92,7 @@ gtk_handle_box_get_type (void)
 
   if (!handle_box_type)
     {
-      GtkTypeInfo handle_box_info =
+      static const GtkTypeInfo handle_box_info =
       {
 	"GtkHandleBox",
 	sizeof (GtkHandleBox),
@@ -259,8 +259,10 @@ gtk_handle_box_map (GtkWidget *widget)
   bin = GTK_BIN (widget);
   hb = GTK_HANDLE_BOX (widget);
 
-  gdk_window_show (hb->bin_window);
-  gdk_window_show (widget->window);
+  if (bin->child &&
+      GTK_WIDGET_VISIBLE (bin->child) &&
+      !GTK_WIDGET_MAPPED (bin->child))
+    gtk_widget_map (bin->child);
 
   if (hb->child_detached && !hb->float_window_mapped)
     {
@@ -268,10 +270,8 @@ gtk_handle_box_map (GtkWidget *widget)
       hb->float_window_mapped = TRUE;
     }
 
-  if (bin->child &&
-      GTK_WIDGET_VISIBLE (bin->child) &&
-      !GTK_WIDGET_MAPPED (bin->child))
-    gtk_widget_map (bin->child);
+  gdk_window_show (hb->bin_window);
+  gdk_window_show (widget->window);
 }
 
 static void
@@ -575,8 +575,8 @@ gtk_handle_box_size_allocate (GtkWidget     *widget,
 	}
       else
 	{
-	  child_allocation.width = MAX (1, widget->allocation.width - 2 * border_width);
-	  child_allocation.height = MAX (1, widget->allocation.height - 2 * border_width);
+	  child_allocation.width = MAX (1, (gint)widget->allocation.width - 2 * border_width);
+	  child_allocation.height = MAX (1, (gint)widget->allocation.height - 2 * border_width);
 
 	  if (hb->handle_position == GTK_POS_LEFT ||
 	      hb->handle_position == GTK_POS_RIGHT)
