@@ -27,7 +27,7 @@
 
 enum {
   ARG_0,
-  ARG_ADJUSTMENT,
+  ARG_ADJUSTMENT
 };
 
 static void gtk_hscrollbar_class_init       (GtkHScrollbarClass *klass);
@@ -58,7 +58,7 @@ gtk_hscrollbar_get_type (void)
   
   if (!hscrollbar_type)
     {
-      GtkTypeInfo hscrollbar_info =
+      static const GtkTypeInfo hscrollbar_info =
       {
         "GtkHScrollbar",
         sizeof (GtkHScrollbar),
