@@ -29,6 +29,7 @@
 #include "gtkmain.h"
 #include "gtkpreview.h"
 #include "gtkrc.h"
+#include "gtkscrolledwindow.h"
 #include "gtkselection.h"
 #include "gtksignal.h"
 #include "gtktable.h"
@@ -149,17 +150,17 @@ gtk_check_version (guint required_major,
 		   guint required_micro)
 {
   if (required_major > GTK_MAJOR_VERSION)
-    return "Gtk+ version to old (major mismatch)";
+    return "Gtk+ version too old (major mismatch)";
   if (required_major < GTK_MAJOR_VERSION)
-    return "Gtk+ version to new (major mismatch)";
+    return "Gtk+ version too new (major mismatch)";
   if (required_minor > GTK_MINOR_VERSION)
-    return "Gtk+ version to old (minor mismatch)";
+    return "Gtk+ version too old (minor mismatch)";
   if (required_minor < GTK_MINOR_VERSION)
-    return "Gtk+ version to new (minor mismatch)";
+    return "Gtk+ version too new (minor mismatch)";
   if (required_micro < GTK_MICRO_VERSION - GTK_BINARY_AGE)
-    return "Gtk+ version to new (micro mismatch)";
+    return "Gtk+ version too new (micro mismatch)";
   if (required_micro > GTK_MICRO_VERSION)
-    return "Gtk+ version to old (micro mismatch)";
+    return "Gtk+ version too old (micro mismatch)";
   return NULL;
 }
 
@@ -629,7 +630,7 @@ gtk_main_do_event (GdkEvent *event)
     {
       grab_widget = event_widget;
     }
-  
+
   /* Not all events get sent to the grabbing widget.
    * The delete, destroy, expose, focus change and resize
    *  events still get sent to the event widget because
@@ -675,7 +676,49 @@ gtk_main_do_event (GdkEvent *event)
     case GDK_VISIBILITY_NOTIFY:
       gtk_widget_event (event_widget, event);
       break;
-      
+
+    case GDK_BUTTON_PRESS:
+    case GDK_2BUTTON_PRESS:
+    case GDK_3BUTTON_PRESS:
+    /* We treat button 4-5 specially, assume we have
+     * a MS-style scrollwheel mouse, and try to find
+     * a plausible widget to scroll. We also trap
+     * button 4-5 double and triple clicks here, since
+     * they will be generated if the user scrolls quickly.
+     */
+      if ((grab_widget == event_widget) &&
+	  (event->button.button == 4 || event->button.button == 5))
+	{
+	  GtkWidget *range = NULL;
+	  GtkWidget *scrollwin;
+	  
+	  if (GTK_IS_RANGE (event_widget))
+	    range = event_widget;
+	  else
+	    {
+	      scrollwin = gtk_widget_get_ancestor (event_widget,
+						   GTK_TYPE_SCROLLED_WINDOW);
+	      if (scrollwin)
+		range = GTK_SCROLLED_WINDOW (scrollwin)->vscrollbar;
+	    }
+	  
+	  if (range && GTK_WIDGET_VISIBLE (range))
+	    {
+	      if (event->type == GDK_BUTTON_PRESS)
+		{
+		  GtkAdjustment *adj = GTK_RANGE (range)->adjustment;
+		  gfloat new_value = adj->value + ((event->button.button == 4) ? 
+						   -adj->page_increment / 2: 
+						    adj->page_increment / 2);
+		  new_value = CLAMP (new_value, adj->lower, adj->upper - adj->page_size);
+		  gtk_adjustment_set_value (adj, new_value);
+		}
+	      break;
+	    }
+	}
+      gtk_propagate_event (grab_widget, event);
+      break;
+
     case GDK_KEY_PRESS:
     case GDK_KEY_RELEASE:
       if (key_snoopers)
@@ -685,9 +728,6 @@ gtk_main_do_event (GdkEvent *event)
 	}
       /* else fall through */
     case GDK_MOTION_NOTIFY:
-    case GDK_BUTTON_PRESS:
-    case GDK_2BUTTON_PRESS:
-    case GDK_3BUTTON_PRESS:
     case GDK_BUTTON_RELEASE:
     case GDK_PROXIMITY_IN:
     case GDK_PROXIMITY_OUT:
