@@ -62,7 +62,7 @@ gtk_progress_get_type (void)
 
   if (!progress_type)
     {
-      GtkTypeInfo progress_info =
+      static const GtkTypeInfo progress_info =
       {
 	"GtkProgress",
 	sizeof (GtkProgress),
@@ -452,7 +452,7 @@ gtk_progress_set_adjustment (GtkProgress   *progress,
   if (adjustment)
     g_return_if_fail (GTK_IS_ADJUSTMENT (adjustment));
   else
-    adjustment = (GtkAdjustment*) gtk_object_new (GTK_TYPE_ADJUSTMENT, NULL);
+    adjustment = (GtkAdjustment*) gtk_adjustment_new (0, 0, 100, 0, 0, 0);
 
   if (progress->adjustment != adjustment)
     {
@@ -475,10 +475,10 @@ gtk_progress_set_adjustment (GtkProgress   *progress,
 }
 
 void
-gtk_progress_reconfigure (GtkProgress *progress,
-			  gfloat value,
-			  gfloat min,
-			  gfloat max)
+gtk_progress_configure (GtkProgress *progress,
+			gfloat value,
+			gfloat min,
+			gfloat max)
 {
   GtkAdjustment *adj;
   gboolean changed = FALSE;
