@@ -141,11 +141,8 @@ void		gtk_spin_button_set_shadow_type	   (GtkSpinButton  *spin_button,
 
 void		gtk_spin_button_set_snap_to_ticks  (GtkSpinButton  *spin_button,
 						    gboolean	    snap_to_ticks);
-/* deprecated */
-void		gtk_spin_button_construct	   (GtkSpinButton  *spin_button,
-						    GtkAdjustment  *adjustment,
-						    gfloat	    climb_rate,
-						    guint	    digits);
+void            gtk_spin_button_update             (GtkSpinButton  *spin_button);
+
 
 
 #ifdef __cplusplus
