@@ -51,7 +51,8 @@ typedef enum
   GTK_RC_STYLE	       = 1 << 16,
   GTK_COMPOSITE_CHILD  = 1 << 17,
   GTK_NO_REPARENT      = 1 << 18,
-  GTK_APP_PAINTABLE    = 1 << 19
+  GTK_APP_PAINTABLE    = 1 << 19,
+  GTK_RECEIVES_DEFAULT = 1 << 20
 } GtkWidgetFlags;
 
 /* Macro for casting a pointer to a GtkWidget or GtkWidgetClass pointer.
@@ -90,6 +91,7 @@ typedef enum
 #define GTK_WIDGET_RC_STYLE(wid)	  ((GTK_WIDGET_FLAGS (wid) & GTK_RC_STYLE) != 0)
 #define GTK_WIDGET_COMPOSITE_CHILD(wid)	  ((GTK_WIDGET_FLAGS (wid) & GTK_COMPOSITE_CHILD) != 0)
 #define GTK_WIDGET_APP_PAINTABLE(wid)	  ((GTK_WIDGET_FLAGS (wid) & GTK_APP_PAINTABLE) != 0)
+#define GTK_WIDGET_RECEIVES_DEFAULT(wid)  ((GTK_WIDGET_FLAGS (wid) & GTK_RECEIVES_DEFAULT) != 0)
   
 /* Macros for setting and clearing widget flags.
  */
@@ -526,6 +528,19 @@ GtkWidget*   gtk_widget_get_ancestor	(GtkWidget	*widget,
 					 GtkType	widget_type);
 GdkColormap* gtk_widget_get_colormap	(GtkWidget	*widget);
 GdkVisual*   gtk_widget_get_visual	(GtkWidget	*widget);
+
+/* The following functions must not be called on an already
+ * realized widget. Because it is possible that somebody
+ * can call get_colormap() or get_visual() and save the
+ * result, these functions are probably only safe to
+ * call in a widget's init() function.
+ */
+void         gtk_widget_set_colormap    (GtkWidget      *widget,
+					 GdkColormap    *colormap);
+void         gtk_widget_set_visual      (GtkWidget      *widget, 
+					 GdkVisual      *visual);
+
+
 gint	     gtk_widget_get_events	(GtkWidget	*widget);
 void	     gtk_widget_get_pointer	(GtkWidget	*widget,
 					 gint		*x,
